"""
Base command
"""
import uuid

from dataclasses import dataclass
from html import escape

import pyepp.helper as helper  # pylint: disable=consider-using-from-import
from pyepp.epp import EppCommunicator, EppResultData
from pyepp.command_templates import template_engine


class ErrorCodeInResultException(Exception):
    """
    Error code in result exception
    """


# pylint: disable=too-few-public-methods
class BaseCommand:
    """
    Base command class. Other EPP commands will inherit this class.
    """
    PARAMS = ()

    def __init__(self, epp_communicator: EppCommunicator) -> None:
        """
        :param epp_communicator: EPP Communicator object
        """
        self._epp_communicator = epp_communicator

    def execute(self, xml_command: str, **kwargs) -> EppResultData:
        """This receives an EPP XML command and the arguments and send to the EPP server to be executed.

        :param xml_command: XML command
        :param kwargs: Keyword arguments

        :return: Response Object
        """
        cmd = self._prepare_command(xml_command, **kwargs)

        result = self._epp_communicator.execute(cmd)
        return result

    def _prepare_command(self, cmd: str, **kwargs: str) -> str:
        """Prepare an EPP XML command for execution by setting up the arguments.

        :param cmd: Command in XML format
        :param kwargs: Keyword arguments

        :return: XML command
        """
        if cmd.find('client_transaction_id') != -1 and not kwargs.get('client_transaction_id'):
            kwargs['client_transaction_id'] = str(uuid.uuid4())

        if cmd.find('password') != -1 and not kwargs.get('password'):
            kwargs['password'] = helper.generate_password(16)

        kwargs = {key: value for key, value in kwargs.items() if value}

        new_kwargs = self.__escape_dict(kwargs)

        template = template_engine.from_string(cmd)
        xml = template.render(**new_kwargs)

        return xml

    def __escape_list(self, input_list: list) -> list:
        result = []

        for value in input_list:
            if isinstance(value, list):
                result.append(self.__escape_list(value))
            elif isinstance(value, dict):
                result.append(self.__escape_dict(value))
            elif isinstance(value, str):
                result.append(escape(value))
            elif value:
                result.append(value)

        return result

    def __escape_dict(self, input_dict: dict) -> dict:
        result = {}

        for key, value in input_dict.items():
            if isinstance(value, list):
                result[key] = self.__escape_list(value)
            elif isinstance(value, dict):
                result[key] = self.__escape_dict(value)
            elif isinstance(value, str):
                result[key] = escape(value)
            elif value:
                result[key] = value

        return result

    def _data_to_dict(self, data: dataclass) -> dict:
        """Convert a dataclass to a dict.

        :param data: data

        :return: data
        """



"""
Contact Mapping Module. This module is used to manage contact objects in Registry.
"""

from typing import Optional
from dataclasses import dataclass, asdict
from bs4 import BeautifulSoup

from pyepp.base_command import BaseCommand
from pyepp.command_templates import (
    CONTACT_CHECK_XML,
    CONTACT_INFO_XML,
    CONTACT_CREAT_XML,
    CONTACT_DELETE_XML,
    CONTACT_UPDATE_XML,
)
from pyepp.epp import EppResultCode, EppResultData


@dataclass
class AddressData:
    """Contact address data class."""

    street_1: Optional[str] = ""
    city: Optional[str] = ""
    country_code: Optional[str] = ""
    street_2: Optional[str] = ""
    street_3: Optional[str] = ""
    province: Optional[str] = ""
    postal_code: Optional[str] = ""


@dataclass
class PostalInfoData:
    """Contact postal info data class."""

    name: Optional[str]
    organization: Optional[str] = ""
    address: Optional[AddressData] = None


@dataclass
class ContactData:
    """Contact data class. Contains the properties of the contacts associated with the domain name."""

    # pylint: disable=invalid-name,too-many-instance-attributes
    id: str
    email: Optional[str] = ""
    postal_info: Optional[PostalInfoData] = None
    status: Optional[list[str]] = None
    phone: Optional[str] = ""
    fax: Optional[str] = ""
    password: Optional[str] = ""
    create_date: Optional[str] = ""
    creat_client_id: Optional[str] = ""
    sponsoring_client_id: Optional[str] = ""
    update_client_id: Optional[str] = ""
    update_date: Optional[str] = ""


class Contact(BaseCommand):
    """Epp Contact object class. This class is used to create and manage the contacts within the Registry.

    Contacts are individuals or organizations that are associated with domain names. There are different types of
    contacts including:

    * Registrant - The entity that has the authority to use and manage the domain name.
    * Administrative Contact - Either the Registrant or someone authorized to act on behalf of the Registrant.
    * Technical Contact - A technical contact is an individual identified as a contact for technical information-related
      administration of a registered domain name.
    * Billing Contact - Also known as the Finance Contact, this is the individual or organization responsible for
      payment of fees related to the domain name and will monitor period activity, account balances, and account status.
    """

    def _data_to_dict(self, data: ContactData) -> dict:
        """Convert a contact dataclass to a dictionary.

        :param data: Contact details

        :return: Contact details
        :rtype: dict
        """
        data_dict = asdict(data)
        postal_info = data_dict.pop("postal_info", {})
        address = postal_info.pop("address", {})

        if address:
            data_dict.update(address)
        if postal_info:
            data_dict.update(postal_info)

        return data_dict

    def check(
        self, contact_ids: list[str], client_transaction_id: Optional[str] = None
    ) -> EppResultData:
        """A successful Contact Check request determines whether a Contact ID is available for use and whether a contact
        can be created in the Registry. When creating a new contact, the Registrar must generate a Registry-unique
        contact ID. A Registry Contact Check request can determine whether an ID is already in use.

        :param contact_ids: List of contact ids
        :param client_transaction_id: Client transaction id

        :return: contact check result
        :rtype: EppResultData
        """
        result = self.execute(
            CONTACT_CHECK_XML,
            ids=contact_ids,
            client_transaction_id=client_transaction_id,
        )

        if result.code != int(EppResultCode.SUCCESS.value):
            return result

        raw_response = BeautifulSoup(result.raw_response, "xml")
        contacts_check_data = raw_response.find_all("cd")

        result_data = {}
        for contact_cd in contacts_check_data:
            contact = contact_cd.find("id")
            available = contact.get("avail") in ("true", "1")
            reason = contact_cd.find("reason").text if not available else None
            result_data[contact.text] = {
                "avail": available,
                "reason": reason,
            }

        result.result_data = result_data

        return result

    def info(
        self, contact_id: str, client_transaction_id: Optional[str] = None
    ) -> EppResultData:
        """
        A successful Contact Info request retrieves information associated with an existing contact.
        All available information is returned if the querying Registrar is the contact’s sponsor. For a non-sponsoring
        Registrar, all contact information is returned if the correct authorization code is entered. As well, if the
        Authorization Code Expiry has been configured, the authorization code must not be expired. Otherwise,
        the <contact:info> will fail.

        :param contact_id: Contact ID
        :param client_transaction_id: Client transaction id

        :return: Contact details
        :rtype: EppResultData
        """
        result = self.execute(
            CONTACT_INFO_XML, id=contact_id, client_transaction_id=client_transaction_id
        )

        if result.code != int(EppResultCode.SUCCESS.value):
            return result

        raw_response = BeautifulSoup(result.raw_response, "xml")

        result_data = {
            "id": raw_response.find("id").text,
            "status": [status.text for status in raw_response.find_all("status")],
            "create_date": raw_response.find("crDate").text,
            "creat_client_id": raw_response.find("crID").text,
            "sponsoring_client_id": raw_response.find("clID").text,
            "update_client_id": (
                raw_response.find("upID").text if raw_response.find("upID") else None
            ),
            "update_date": (
                raw_response.find("upDate").text
                if raw_response.find("upDate")
                else None
            ),
            "postal_info": PostalInfoData(
                **{
                    "name": raw_response.find("name").text,
                    "organization": (
                        raw_response.find("org").text
                        if raw_response.find("org")
                        else None
                    ),
                    "address": AddressData(
                        **{
                            "street_1": (
                                raw_response.find_all("street")[0].text
                                if len(raw_response.find_all("street")) >= 1
                                else None
                            ),
                            "street_2": (
                                raw_response.find_all("street")[1].text
                                if len(raw_response.find_all("street")) >= 2
                                else None
                            ),
                            "street_3": (
                                raw_response.find_all("street")[2].text
                                if len(raw_response.find_all("street")) >= 3
                                else None
                            ),
                            "city": raw_response.find("city").text,
                            "province": (
                                raw_response.find("sp").text
                                if raw_response.find("sp")
                                else None
                            ),
                            "postal_code": (
                                raw_response.find("pc").text
                                if raw_response.find("pc")
                                else None
                            ),
                            "country_code": raw_response.find("cc").text,
                        }
                    ),
                }
            ),
            "phone": (
                raw_response.find("voice").text if raw_response.find("voice") else None
            ),
            "fax": raw_response.find("fax").text if raw_response.find("fax") else None,
            "email": raw_response.find("email").text,
        }

        if raw_response.find("pw"):
            result_data["password"] = raw_response.find("pw").text

        result.result_data = ContactData(**result_data)

        return result

    def create(
        self, contact: ContactData, client_transaction_id: Optional[str] = None
    ) -> EppResultData:
        """A successful Contact Create request creates a contact object in the Registry. To create a domain name
        successfully, a Registrar does not need to be the sponsor of the related hosts but must be the sponsor of all
        assigned contacts.

        :param contact: Contact
        :param client_transaction_id: Client transaction id

        :return: Result object
        :rtype: EppResultData
        """
        params = self._data_to_dict(contact)
        params["client_transaction_id"] = client_transaction_id

        result = self.execute(CONTACT_CREAT_XML, **params)

        return result

    def delete(
        self, contact_id: str, client_transaction_id: Optional[str] = None
    ) -> EppResultData:
        """A successful Contact Delete request deletes a contact object from the Registry

        :param contact_id: Contact ID
        :param client_transaction_id: Client transaction id

        :return: Result object
        """
        result = self.execute(
            CONTACT_DELETE_XML,
            id=contact_id,
            client_transaction_id=client_transaction_id,
        )

        return result

    def update(
        self,
        contact: ContactData,
        add_status: Optional[str] = "",
        remove_status: Optional[str] = "",
        client_transaction_id: Optional[str] = None,
    ) -> EppResultData:
        """A successful Contact Update request modifies a contact object in the Registry. Updates to Registrant contacts
        must be valid and must be complete.

        :param contact: Contact details to be updated
        :param add_status: Status to be added
        :param remove_status: Status to be removed
        :param client_transaction_id: Client transaction id

        :return: Result object
        :rtype: EppResultData
        """

        params = self._data_to_dict(contact)

        params["add_status"] = add_status
        params["remove_status"] = remove_status

        params["postalinfo_change"] = bool(contact.postal_info)
        params["address_change"] = bool(contact.postal_info.address)
        params["client_transaction_id"] = client_transaction_id

        result = self.execute(CONTACT_UPDATE_XML, **params)

        return result



"""
Contact unit tests
"""
import unittest
from unittest.mock import MagicMock

from pyepp.contact import ContactData, PostalInfoData, AddressData, Contact
from pyepp.epp import EppCommunicator, EppResultData


class ContactTest(unittest.TestCase):
    """
    Contact unit tests
    """

    def setUp(self) -> None:
        self.maxDiff = None

    def test_data_to_dict(self) -> None:
        data = ContactData(
            id='id',
            status=['status1', 'status2'],
            create_date='create_date',
            creat_client_id='creat_client_id',
            sponsoring_client_id='sponsoring_client_id',
            update_client_id='update_client_id',
            update_date='update_date',
            postal_info=PostalInfoData(
                name='name',
                organization='organization',
                address=AddressData(
                    street_1='street_1',
                    street_2='street_2',
                    street_3='street_3',
                    city='city',
                    province='province',
                    postal_code='postal_code',
                    country_code='country_code',
                ),
            ),
            phone='phone',
            fax='fax',
            email='email',
            password='',
        )

        expected_result = {
            'id': 'id',
            'status': ['status1', 'status2'],
            'create_date': 'create_date',
            'creat_client_id': 'creat_client_id',
            'sponsoring_client_id': 'sponsoring_client_id',
            'update_client_id': 'update_client_id',
            'update_date': 'update_date',
            'name': 'name',
            'organization': 'organization',
            'street_1': 'street_1',
            'street_2': 'street_2',
            'street_3': 'street_3',
            'city': 'city',
            'province': 'province',
            'postal_code': 'postal_code',
            'country_code': 'country_code',
            'phone': 'phone',
            'fax': 'fax',
            'email': 'email',
            'password': '',
        }

        epp_communicator = MagicMock(EppCommunicator)
        contact = Contact(epp_communicator)

        result = contact._data_to_dict(data)
        self.assertDictEqual(result, expected_result)

    def test_check_unsuccessful(self) -> None:
        epp_communicator = MagicMock(EppCommunicator)
        contact = Contact(epp_communicator)
        expected_result = \
            EppResultData(**{'code': 2000, 'message': "Command completed unsuccessfully",
                             'reason': None, 'raw_response': "response", 'result_data': None})
        contact.execute = MagicMock(return_value=expected_result)

        result = contact.check(['contact1'])

        self.assertEqual(result, expected_result)

    def test_check(self) -> None:
        epp_communicator = MagicMock(EppCommunicator)
        contact = Contact(epp_communicator)
        expected_result = EppResultData(**{
            'client_transaction_id': 'eccb044f-a80f-4db3-a918-988f1ac918e3',
            'code': 1000,
            'message': 'Command completed successfully',
            'raw_response': '<response>\n'
                            '<result code="1000">\n'
                            '<msg>Command completed successfully</msg>\n'
                            '</result>\n'
                            '<resData>\n'
                            '<contact:chkData>\n'
                            '<contact:cd>\n'
                            '<contact:id avail="true">contact1</contact:id>\n'
                            '</contact:cd>\n'
                            '</contact:chkData>\n'
                            '</resData>\n'
                            '<trID>\n'
                            '<clTRID>eccb044f-a80f-4db3-a918-988f1ac918e3</clTRID>\n'
                            '<svTRID>CIRA-000062206323-0000000003</svTRID>\n'
                            '</trID>\n'
                            '</response>',
            'reason': None,
            'repository_object_id': None,
            'result_data': {'contact1': {'avail': True, 'reason': None},
                            'contact2': {'avail': False,
                                         'reason': 'Selected contact ID is not '
                                                   'available'}},
            'server_transaction_id': 'CIRA-000062206323-0000000003'
        })

        contact.execute = MagicMock(return_value=expected_result)

        result = contact.check(['contact1', 'contact2'])

        self.assertEqual(result, expected_result)

    def test_info_unsuccessful(self) -> None:
        epp_communicator = MagicMock(EppCommunicator)
        contact = Contact(epp_communicator)
        expected_result = \
            EppResultData(**{'code': 2000, 'message': "Command completed unsuccessfully",
                             'reason': None, 'raw_response': "response", 'result_data': None})
        contact.execute = MagicMock(return_value=expected_result)

        result = contact.info('contact1')

        self.assertEqual(result, expected_result)

    def test_info(self) -> None:
        epp_communicator = MagicMock(EppCommunicator)
        contact = Contact(epp_communicator)
        execute_result = EppResultData(**{
            'client_transaction_id': '5123c3d4-79ce-4d87-ad7b-d234eb992474',
            'code': 1000,
            'message': 'Command completed successfully',
            'raw_response': '<response>\n'
                            '<result code="1000">\n'
                            '<msg>Command completed successfully</msg>\n'
                            '</result>\n'
                            '<resData>\n'
                            '<contact:infData>\n'
                            '<contact:id>inz-contact-1</contact:id>\n'
                            '<contact:roid>9175701-INZ</contact:roid>\n'
                            '<contact:status s="linked"/>\n'
                            '<contact:postalInfo type="loc">\n'
                            '<contact:name>inz</contact:name>\n'
                            '<contact:addr>\n'
                            '<contact:street>18 portmore place</contact:street>\n'
                            '<contact:city>Wellington</contact:city>\n'
                            '<contact:cc>NZ</contact:cc>\n'
                            '</contact:addr>\n'
                            '</contact:postalInfo>\n'
                            '<contact:email>inz@internet.net.nz</contact:email>\n'
                            '<contact:clID>933</contact:clID>\n'
                            '<contact:crID>933</contact:crID>\n'
                            '<contact:crDate>2023-02-23T02:59:16.784Z</contact:crDate>\n'
                            '<contact:upID>CIRA_RAR_1</contact:upID>\n'
                            '<contact:upDate>2023-02-23T21:59:01.021Z</contact:upDate>\n'
                            '<contact:authInfo>\n'
                            '<contact:pw>PassWord</contact:pw>\n'
                            '</contact:authInfo>\n'
                            '</contact:infData>\n'
                            '</resData>\n'
                            '<trID>\n'
                            '<clTRID>5123c3d4-79ce-4d87-ad7b-d234eb992474</clTRID>\n'
                            '<svTRID>CIRA-000062211375-0000000003</svTRID>\n'
                            '</trID>\n'
                            '</response>',
            'reason': None,
            'repository_object_id': '9175701-INZ',
            'server_transaction_id': 'CIRA-000062211375-0000000003',
            'result_data': None
        })

        expected_result = EppResultData(**{
            'client_transaction_id': '5123c3d4-79ce-4d87-ad7b-d234eb992474',
            'code': 1000,
            'message': 'Command completed successfully',
            'raw_response': '<response>\n'
                            '<result code="1000">\n'
                            '<msg>Command completed successfully</msg>\n'
                            '</result>\n'
                            '<resData>\n'
                            '<contact:infData>\n'
                            '<contact:id>inz-contact-1</contact:id>\n'
                            '<contact:roid>9175701-INZ</contact:roid>\n'
                            '<contact:status s="linked"/>\n'
                            '<contact:postalInfo type="loc">\n'
                            '<contact:name>inz</contact:name>\n'
                            '<contact:addr>\n'
                            '<contact:street>18 portmore place</contact:street>\n'
                            '<contact:city>Wellington</contact:city>\n'
                            '<contact:cc>NZ</contact:cc>\n'
                            '</contact:addr>\n'
                            '</contact:postalInfo>\n'
                            '<contact:email>inz@internet.net.nz</contact:email>\n'
                            '<contact:clID>933</contact:clID>\n'
                            '<contact:crID>933</contact:crID>\n'
                            '<contact:crDate>2023-02-23T02:59:16.784Z</contact:crDate>\n'
                            '<contact:upID>CIRA_RAR_1</contact:upID>\n'
                            '<contact:upDate>2023-02-23T21:59:01.021Z</contact:upDate>\n'
                            '<contact:authInfo>\n'
                            '<contact:pw>PassWord</contact:pw>\n'
                            '</contact:authInfo>\n'
                            '</contact:infData>\n'
                            '</resData>\n'
                            '<trID>\n'
                            '<clTRID>5123c3d4-79ce-4d87-ad7b-d234eb992474</clTRID>\n'
                            '<svTRID>CIRA-000062211375-0000000003</svTRID>\n'
                            '</trID>\n'
                            '</response>',
            'reason': None,
            'repository_object_id': '9175701-INZ',
            'result_data': ContactData(id='inz-contact-1',
                                       email='inz@internet.net.nz',
                                       postal_info=PostalInfoData(**{'address': AddressData(**{'city': 'Wellington',
                                                                                               'country_code': 'NZ',
                                                                                               'postal_code': None,
                                                                                               'province': None,
                                                                                               'street_1': '18 portmore '
                                                                                                           'place',
                                                                                               'street_2': None,
                                                                                               'street_3': None}),
                                                                     'name': 'inz',
                                                                     'organization': None}),
                                       status=[''],
                                       phone=None,
                                       fax=None,
                                       password='PassWord',
                                       create_date='2023-02-23T02:59:16.784Z',
                                       creat_client_id='933',
                                       sponsoring_client_id='933',
                                       update_client_id='CIRA_RAR_1',
                                       update_date='2023-02-23T21:59:01.021Z'
                                       ),
            'server_transaction_id': 'CIRA-000062211375-0000000003'
        })

        contact.execute = MagicMock(return_value=execute_result)

        result = contact.info('inz-contact-1')

        self.assertEqual(result, expected_result)

    def test_create(self) -> None:
        expected_result = EppResultData(**{'client_transaction_id': 'caae2895-fe01-4f1c-a892-115b17315acc',
                                           'code': 1000,
                                           'message': 'Command completed successfully',
                                           'raw_response': '<response>\n'
                                                           '<result code="1000">\n'
                                                           '<msg>Command completed successfully</msg>\n'
                                                           '</result>\n'
                                                           '<resData>\n'
                                                           '<contact:creData>\n'
                                                           '<contact:id>inz-contact-1</contact:id>\n'
                                                           '<contact:crDate>2023-04-26T23:06:11.894Z</contact:crDate>\n'
                                                           '</contact:creData>\n'
                                                           '</resData>\n'
                                                           '<trID>\n'
                                                           '<clTRID>caae2895-fe01-4f1c-a892-115b17315acc</clTRID>\n'
                                                           '<svTRID>CIRA-000062214171-0000000003</svTRID>\n'
                                                           '</trID>\n'
                                                           '</response>',
                                           'reason': None,
                                           'repository_object_id': None,
                                           'server_transaction_id': 'CIRA-000062214171-0000000003',
                                           'result_data': None})

        create_params = ContactData(
            id='inz-contact-1',
            email='epp@internetnz.net.nz',
            postal_info=PostalInfoData(
                name='IRS EPP',
                organization='INZ',
                address=AddressData(
                    street_1='18 Willis Street',
                    street_2='Wellington CBD',
                    city='Wellington',
                    country_code='NZ',
                    province='Wellington',
                    postal_code='6011'
                ),
            ),
            phone='+64.111111111'
        )

        epp_communicator = MagicMock(EppCommunicator)
        contact = Contact(epp_communicator)
        contact.execute = MagicMock(return_value=expected_result)

        result = contact.create(create_params)

        self.assertEqual(result, expected_result)

    def test_delete(self) -> None:
        expected_result = EppResultData(**{'client_transaction_id': 'a21a659d-5040-4848-9f5f-0cffa0ff62d1',
                                           'code': 1000,
                                           'message': 'Command completed successfully',
                                           'raw_response': '<response>\n'
                                                           '<result code="1000">\n'
                                                           '<msg>Command completed successfully</msg>\n'
                                                           '</result>\n'
                                                           '<trID>\n'
                                                           '<clTRID>a21a659d-5040-4848-9f5f-0cffa0ff62d1</clTRID>\n'
                                                           '<svTRID>CIRA-000062220522-0000000004</svTRID>\n'
                                                           '</trID>\n'
                                                           '</response>',
                                           'reason': None,
                                           'repository_object_id': None,
                                           'server_transaction_id': 'CIRA-000062220522-0000000004',
                                           'result_data': None})

        epp_communicator = MagicMock(EppCommunicator)
        contact = Contact(epp_communicator)
        contact.execute = MagicMock(return_value=expected_result)

        result = contact.delete('inz-contact-1')

        self.assertEqual(result, expected_result)

    def test_update(self) -> None:
        update_params = ContactData(
            id='inz-contact-1',
            email='ehsan@internetnz.net.nz',
            postal_info=PostalInfoData(name='IRS EPP2')
        )

        expected_result = EppResultData(**{'client_transaction_id': '0e872842-b77b-4800-9572-c72e46e068de',
                                           'code': 1000,
                                           'message': 'Command completed successfully',
                                           'raw_response': '<response>\n'
                                                           '<result code="1000">\n'
                                                           '<msg>Command completed successfully</msg>\n'
                                                           '</result>\n'
                                                           '<trID>\n'
                                                           '<clTRID>0e872842-b77b-4800-9572-c72e46e068de</clTRID>\n'
                                                           '<svTRID>CIRA-000062223355-0000000004</svTRID>\n'
                                                           '</trID>\n'
                                                           '</response>',
                                           'reason': None,
                                           'repository_object_id': None,
                                           'server_transaction_id': 'CIRA-000062223355-0000000004',
                                           'result_data': None})

        epp_communicator = MagicMock(EppCommunicator)
        contact = Contact(epp_communicator)
        contact.execute = MagicMock(return_value=expected_result)

        result = contact.update(update_params)

        self.assertEqual(result, expected_result)



"""
Contact cli module
"""
import click

from pyepp.cli import utils
from pyepp.contact import Contact, ContactData, PostalInfoData, AddressData


@click.group(name='contact')
@click.pass_context
def contact_group(ctx):
    """
    To work with Contact objects in the registry.
    :param ctx:
    :return:
    """
    ctx.obj.registry_object = Contact(ctx.obj.epp)


@click.command(name='info')
@click.argument('contact-id')
@click.option('--client-transaction-id')
@click.pass_context
def contact_info(ctx, contact_id, client_transaction_id) -> None:
    """Returns contact details."""
    result = ctx.obj.info(contact_id, client_transaction_id)
    utils.echo(result)


@click.command(name='check')
@click.argument('contact-ids', nargs=-1)
@click.option('--client-transaction-id')
@click.pass_context
def contact_check(ctx, contact_ids: tuple, client_transaction_id) -> None:
    """Checks if contact(s) exist in the registry."""
    result = ctx.obj.check(list(contact_ids), client_transaction_id)
    utils.echo(result)


@click.command(name='delete')
@click.argument('contact-id')
@click.option('--client-transaction-id')
@click.pass_context
def contact_delete(ctx, contact_id, client_transaction_id) -> None:
    """Deletes a given contact from registry."""
    result = ctx.obj.delete(contact_id, client_transaction_id)
    utils.echo(result)


@click.command(name='update')
@click.argument('contact-id')
@click.option('--email')
@click.option('--name')
@click.option('--organization')
@click.option('--street-1')
@click.option('--street-2')
@click.option('--street-3')
@click.option('--city', help="required if postal info gets updated.")
@click.option('--province')
@click.option('--postal-code')
@click.option('--country-code', help="required if postal info gets updated.")
@click.option('--phone')
@click.option('--fax')
@click.option('--password')
@click.option('--add-status')
@click.option('--remove-status')
@click.option('--client-transaction-id')
@click.pass_context
# pylint: disable=too-many-arguments, too-many-locals, too-many-boolean-expressions
def contact_update(ctx, contact_id, name, organization, street_1, street_2, street_3, city, province, postal_code,
                   country_code, phone, fax, email, password, add_status, remove_status, client_transaction_id) -> None:
    """Updates contact's details.

    CONTACT_ID: Contact id
    """
    contact_to_update = ContactData(
        id=contact_id,
        email=email,
        phone=phone,
        fax=fax,
        password=password,
        postal_info=PostalInfoData(
            name=name,
            organization=organization,
        )
    )

    if street_1 or street_2 or street_3 or city or province or postal_code or country_code:
        contact_to_update.postal_info.address = AddressData(
            street_1=street_1,
            street_2=street_2,
            street_3=street_3,
            city=city,
            province=province,
            postal_code=postal_code,
            country_code=country_code
        )

    result = ctx.obj.update(contact_to_update, add_status, remove_status, client_transaction_id)
    utils.echo(result)


@click.command(name='create')
@click.argument('contact-id')
@click.option('--email', required=True)
@click.option('--name', required=True)
@click.option('--city', required=True)
@click.option('--country-code', required=True)
@click.option('--organization')
@click.option('--street-1')
@click.option('--street-2')
@click.option('--street-3')
@click.option('--province')
@click.option('--postal-code')
@click.option('--phone')
@click.option('--fax')
@click.option('--password')
@click.option('--client-transaction-id')
@click.pass_context
# pylint: disable=too-many-arguments, too-many-locals
def contact_create(ctx, contact_id, name, organization, street_1, street_2, street_3, city, province, postal_code,
                   country_code, phone, fax, email, password, client_transaction_id) -> None:
    """Creates a new contact in the registry.

    CONTACT_ID: A unique contact id
    """
    contact_to_create = ContactData(
        id=contact_id,
        email=email,
        phone=phone,
        fax=fax,
        password=password,
        postal_info=PostalInfoData(
            name=name,
            organization=organization,
            address=AddressData(
                street_1=street_1,
                street_2=street_2,
                street_3=street_3,
                city=city,
                province=province,
                postal_code=postal_code,
                country_code=country_code
            )
        )
    )

    result = ctx.obj.create(contact_to_create, client_transaction_id)
    utils.echo(result)


contact_group.add_command(contact_check)
contact_group.add_command(contact_info)
contact_group.add_command(contact_delete)
contact_group.add_command(contact_update)
contact_group.add_command(contact_create)



"""
Utilities module
"""
from typing import Optional, Any

import click

OUTPUT_FILE = None


def echo(message: Optional[Any] = None,
         new_line: bool = True,
         err: bool = False,
         color: Optional[bool] = None) -> None:
    """

    :param message: The string or bytes to output. Other objects are
        converted to strings.
    :param err: Write to ``stderr`` instead of ``stdout``.
    :param new_line: Print a newline after the message. Enabled by default.
    :param color: Force showing or hiding colors and other styles. By
        default, Click will remove color if the output does not look like
        an interactive terminal.
    """
    # Convert the string to bytes if writing to a file
    if OUTPUT_FILE:
        message = str.encode(message)

    click.echo(message, file=OUTPUT_FILE, nl=new_line, err=err, color=color)



"""
Executable cli module.
"""
import os
from configparser import RawConfigParser
import logging

import click

from pyepp.cli.host import host_group
from pyepp.cli import cli
from pyepp.cli.contact import contact_group
from pyepp.cli.domain import domain_group
from pyepp.cli.poll import poll_group
from pyepp.cli import utils

logging.basicConfig(level=logging.ERROR)

CONTEXT_SETTINGS = {'help_option_names': ['-h', '--help']}

APP_NAME = 'pyepp'


def load_config():
    """
    Loads the configuration file into context settings. It reads the config file
    from the below paths depends on the host OS.

    Mac OS X (POSIX) and Unix (POSIX):
      ~/.pyepp/config.ini
    Windows (not roaming):
      C:\\Users\\<user>\\AppData\\Local\\pyepp\\config.ini
    """
    cfg = os.path.join(click.get_app_dir(APP_NAME, roaming=False, force_posix=True), 'config.ini')
    parser = RawConfigParser()
    parser.read([cfg])
    config = {}
    for section in parser.sections():
        for key, value in parser.items(section):
            config[f"{section}_{key}".upper()] = value

    CONTEXT_SETTINGS['default_map'] = {
        'server': config.get('PYEPP_SERVER'),
        'port': config.get('PYEPP_PORT'),
        'client_cert': config.get('PYEPP_CLIENT_CERT'),
        'client_key': config.get('PYEPP_CLIENT_KEY'),
        'user': config.get('PYEPP_USER'),
        'password': config.get('PYEPP_PASSWORD')
    }


load_config()


@click.group(name='pyepp', context_settings=CONTEXT_SETTINGS)
@click.option('--server', envvar="PYEPP_SERVER", required=True)
@click.option('--port', envvar="PYEPP_PORT", required=True)
@click.option('--client-cert', envvar="PYEPP_CLIENT_CERT", required=True)
@click.option('--client-key', envvar="PYEPP_CLIENT_KEY", required=True)
@click.option('--user', envvar="PYEPP_USER", required=True)
@click.option('--password', envvar="PYEPP_PASSWORD", required=True)
@click.option('-o', '--output-format', show_default=True, default='XML',
              type=click.Choice(['XML', 'OBJECT', 'MIN'], case_sensitive=False))
@click.option('--no-pretty', is_flag=True, show_default=True, default=False)
@click.option('--dry-run', is_flag=True, show_default=True, default=False)
@click.option('-f', '--file', type=click.File('wb'),
              help='If provided, the output will be written in the file.', default=None)
@click.option('-v', '--verbose', is_flag=True, show_default=True, default=False)
@click.option('-d', '--debug', is_flag=True, show_default=True, default=False)
@click.version_option()
@click.pass_context
# pylint: disable=too-many-arguments
def pyepp_cli(ctx, server, port, client_cert, client_key, user, password, output_format, no_pretty, dry_run,
              file, verbose, debug):
    """A command line interface to work with PyEpp library."""
    ctx.obj = cli.PyEppCli(server, port, client_cert, client_key, user, password, output_format, no_pretty, dry_run)

    if verbose:
        logging.getLogger().setLevel(level=logging.INFO)
    if debug:
        logging.getLogger().setLevel(level=logging.DEBUG)

    utils.OUTPUT_FILE = file


@click.command('run')
@click.argument('xml', type=click.File('rb'))
@click.pass_context
def run_xml(ctx, xml):
    """Receive an XML file containing an EPP XML command and execute it.

    XML: path to an XML file
    """
    xml_command = xml.read()
    result = ctx.obj.execute(xml_command.decode('utf-8'))
    utils.echo(result)


pyepp_cli.add_command(run_xml)
pyepp_cli.add_command(contact_group)
pyepp_cli.add_command(domain_group)
pyepp_cli.add_command(host_group)
pyepp_cli.add_command(poll_group)



"""
EPP Poll Module
"""
from dataclasses import asdict, dataclass
from typing import List, Optional

from bs4 import BeautifulSoup

from pyepp.epp import EppResultData
from pyepp import EppResultCode
from pyepp.base_command import BaseCommand
from pyepp.command_templates import POLL_REQUEST_XML, POLL_ACK_XML


@dataclass
class ServiceMessageData:
    """Service message data class.
    """
    language: str
    message: str


@dataclass
class ServiceMessageQueueData:
    """Service message queue data class.
    """
    message_count: int
    message_id: int
    queue_date: Optional[str] = ''
    messages: Optional[List[ServiceMessageData]] = None


class Poll(BaseCommand):
    """
    Epp Poll class is used to manage registry service messages. It contains poll request and poll acknowledge commands.
    """

    def _data_to_dict(self, data: ServiceMessageQueueData) -> dict:
        """Convert dataclass to dictionary.

        :param ServiceMessageData data: Service message data

        :return: Domain name details
        :rtype: dict
        """
        data_dict = asdict(data)

        return data_dict

    def request(self, client_transaction_id: Optional[str] = None) -> EppResultData:
        """This command is to check and retrieve queued service messages as wel as keep the
        connection alive.

        :param client_transaction_id: Client transaction id

        :return: Result object
        :rtype: EppResultData
        """
        result = self.execute(POLL_REQUEST_XML, client_transaction_id=client_transaction_id)

        if int(result.code) != int(EppResultCode.SUCCESS_ACK_TO_DEQUEUE.value):
            return result

        message_queue = BeautifulSoup(result.raw_response, 'xml')

        result_date = {
            'message_count': int(message_queue.find('msgQ').get('count')),
            'message_id': int(message_queue.find('msgQ').get('id')),
            'queue_date': message_queue.find('qDate').text,
            'messages': [ServiceMessageData(language=message.get('lang'), message=message.text) for message in
                         message_queue.find_all('msg')[1:]] if message_queue.find('msg') else None
        }

        result.result_data = ServiceMessageQueueData(**result_date)

        return result

    def acknowledge(self, message_id: int, client_transaction_id: Optional[str] = None) -> EppResultData:
        """This command will acknowledge and remove a message from the poll queue so that registrars can run another
        poll request to get the next message in line if one exists.

        :param message_id: Message id
        :param client_transaction_id: Client transaction id

        :return: Response object
        :rtype: EppResultData
        """
        result = self.execute(POLL_ACK_XML, message_id=message_id, client_transaction_id=client_transaction_id)

        if int(result.code) != int(EppResultCode.SUCCESS.value):
            return result

        message_queue = BeautifulSoup(result.raw_response, 'xml')

        result_date = {
            'message_count': int(message_queue.find('msgQ').get('count')),
            'message_id': int(message_queue.find('msgQ').get('id')),
        }

        result.result_data = ServiceMessageQueueData(**result_date)

        return result



"""
EPP Poll cli module
"""
import click

from pyepp.poll import Poll
from pyepp.cli import utils


@click.group(name='poll')
@click.pass_context
def poll_group(ctx):
    """ To manage registry service messages."""
    ctx.obj.registry_object = Poll(ctx.obj.epp)


@click.command(name='request')
@click.option('--client-transaction-id')
@click.pass_context
def poll_request(ctx, client_transaction_id):
    """This command is to check and retrieve queued service messages as wel as keep the
        connection alive."""
    result = ctx.obj.request(client_transaction_id=client_transaction_id)
    utils.echo(result)


@click.command(name='acknowledge')
@click.argument('message-id')
@click.option('--client-transaction-id')
@click.pass_context
def poll_acknowledge(ctx, message_id,client_transaction_id):
    """This command will acknowledge and remove a message from the poll queue so that registrars can run another
       poll request to get the next message in line if one exists.

        MESSAGE_ID: Request message id
    """
    result = ctx.obj.acknowledge(message_id,
                                 client_transaction_id=client_transaction_id)
    utils.echo(result)


poll_group.add_command(poll_request)
poll_group.add_command(poll_acknowledge)



"""
Host Name Mapping unit tests
"""
import unittest
from datetime import date
from unittest.mock import MagicMock

from pyepp.poll import Poll, ServiceMessageQueueData, ServiceMessageData
from pyepp.epp import EppCommunicator, EppResultData


class HostTest(unittest.TestCase):
    """
    Host mapping name unit tests
    """

    def setUp(self) -> None:
        self.maxDiff = None

    def test_poll_request_unsuccessful(self) -> None:
        epp_communicator = MagicMock(EppCommunicator)
        poll = Poll(epp_communicator)
        expected_result = \
            EppResultData(**{'code': 2000, 'message': "Command completed unsuccessfully",
                             'reason': None, 'raw_response': "response", 'result_data': None})
        poll.execute = MagicMock(return_value=expected_result)

        result = poll.request()

        self.assertEqual(result, expected_result)

    def test_poll_request(self) -> None:
        epp_communicator = MagicMock(EppCommunicator)
        poll = Poll(epp_communicator)
        expected_result = \
            EppResultData(**{'client_transaction_id': 'c6710aac-cbcf-48d0-9ec3-cfaad1cadc81',
                             'code': 1301,
                             'message': 'Command completed successfully; ack to dequeue',
                             'raw_response': '<response>\n'
                                             '<result code="1301">\n'
                                             '<msg>Command completed successfully; ack to dequeue</msg>\n'
                                             '</result>\n'
                                             '<msgQ count="1" id="27690316">\n'
                                             '<qDate>2023-07-26T00:21:31.246Z</qDate>\n'
                                             '<msg lang="en">Contact inz-contact-1 has been '
                                             'deleted.</msg>\n'
                                             '</msgQ>\n'
                                             '<trID>\n'
                                             '<clTRID>c6710aac-cbcf-48d0-9ec3-cfaad1cadc81</clTRID>\n'
                                             '<svTRID>CIRA-000097025501-0000000002</svTRID>\n'
                                             '</trID>\n'
                                             '</response>',
                             'reason': None,
                             'repository_object_id': None,
                             'result_data': ServiceMessageQueueData(message_count='1',
                                                                    message_id='27690316',
                                                                    queue_date='2023-07-26T00:21:31.246Z',
                                                                    messages=[ServiceMessageData(language='en',
                                                                                                 message='Contact '
                                                                                                         'inz-contact-1 '
                                                                                                         'has '
                                                                                                         'been '
                                                                                                         'deleted.')]),
                             'server_transaction_id': 'CIRA-000097025501-0000000002'})

        poll.execute = MagicMock(return_value=expected_result)

        result = poll.request()

        self.assertEqual(result, expected_result)

    def test_poll_ack_unsuccessful(self) -> None:
        epp_communicator = MagicMock(EppCommunicator)
        poll = Poll(epp_communicator)
        expected_result = \
            EppResultData(**{'code': 1500, 'message': "Command completed successfully",
                             'reason': None, 'raw_response': "response", 'result_data': None})
        poll.execute = MagicMock(return_value=expected_result)

        result = poll.acknowledge(121212)

        self.assertEqual(result, expected_result)

    def test_poll_ack(self) -> None:
        epp_communicator = MagicMock(EppCommunicator)
        poll = Poll(epp_communicator)
        expected_result = \
            EppResultData(**{'client_transaction_id': 'c6710aac-cbcf-48d0-9ec3-cfaad1cadc81',
                             'code': 1301,
                             'message': 'Command completed successfully',
                             'raw_response': '<response>\n'
                                             '<result code="1000">\n'
                                             '<msg>Command completed successfully</msg>\n'
                                             '</result>\n'
                                             '<msgQ count="1" id="27690316">\n'
                                             '</msgQ>\n'
                                             '<trID>\n'
                                             '<clTRID>c6710aac-cbcf-48d0-9ec3-cfaad1cadc81</clTRID>\n'
                                             '<svTRID>CIRA-000097025501-0000000002</svTRID>\n'
                                             '</trID>\n'
                                             '</response>',
                             'reason': None,
                             'repository_object_id': None,
                             'result_data': None,
                             'server_transaction_id': 'CIRA-000097025501-0000000002'})

        poll.execute = MagicMock(return_value=expected_result)

        result = poll.acknowledge(27690316)

        self.assertEqual(result, expected_result)



"""
Domain unit tests
"""
import unittest
from datetime import date
from unittest.mock import MagicMock

from pyepp.domain import Domain, DomainData, DSRecordData, DNSSECAlgorithm, DigestTypeEnum
from pyepp.epp import EppCommunicator, EppResultData


class DomainTest(unittest.TestCase):
    """
    Domain name unit tests
    """

    def setUp(self) -> None:
        self.maxDiff = None

    def test_check_unsuccessful(self) -> None:
        epp_communicator = MagicMock(EppCommunicator)
        domain = Domain(epp_communicator)
        expected_result = \
            EppResultData(**{'code': 2000, 'message': "Command completed unsuccessfully",
                             'reason': None, 'raw_response': "response", 'result_data': None})
        domain.execute = MagicMock(return_value=expected_result)

        result = domain.check(['internet.nz'])

        self.assertEqual(result, expected_result)

    def test_check(self) -> None:
        epp_communicator = MagicMock(EppCommunicator)
        domain = Domain(epp_communicator)
        expected_result = (
            EppResultData(**{'client_transaction_id': '8d537a88-b18c-4f74-903e-aff9be1f902d',
                             'code': 1000,
                             'message': 'Command completed successfully',
                             'raw_response': '<response>\n'
                                             '<result code="1000">\n'
                                             '<msg>Command completed successfully</msg>\n'
                                             '</result>\n'
                                             '<resData>\n'
                                             '<domain:chkData>\n'
                                             '<domain:cd>\n'
                                             '<domain:name avail="false">inz1.nz</domain:name>\n'
                                             '<domain:reason>Registered</domain:reason>\n'
                                             '</domain:cd>\n'
                                             '<domain:cd>\n'
                                             '<domain:name avail="true">inz2.nz</domain:name>\n'
                                             '</domain:cd>\n'
                                             '<domain:cd>\n'
                                             '<domain:name avail="true">inz3.nz</domain:name>\n'
                                             '</domain:cd>\n'
                                             '</domain:chkData>\n'
                                             '</resData>\n'
                                             '<trID>\n'
                                             '<clTRID>8d537a88-b18c-4f74-903e-aff9be1f902d</clTRID>\n'
                                             '<svTRID>CIRA-000062431732-0000000003</svTRID>\n'
                                             '</trID>\n'
                                             '</response>',
                             'reason': None,
                             'repository_object_id': None,
                             'result_data': {'inz3.nz': {'avail': True, 'reason': None},
                                             'inz2.nz': {'avail': True, 'reason': None},
                                             'inz1.nz': {'avail': False, 'reason': 'Registered'}},
                             'server_transaction_id': 'CIRA-000062431732-0000000003'}))

        domain.execute = MagicMock(return_value=expected_result)

        result = domain.check(['inz1.nz', 'inz2.nz', 'inz3.nz'])

        self.assertEqual(result, expected_result)

    def test_info_unsuccessful(self) -> None:
        epp_communicator = MagicMock(EppCommunicator)
        domain = Domain(epp_communicator)
        expected_result = \
            EppResultData(**{'code': 2000, 'message': "Command completed unsuccessfully",
                             'reason': None, 'raw_response': "response", 'result_data': None})
        domain.execute = MagicMock(return_value=expected_result)

        result = domain.info('internet.nz')

        self.assertEqual(result, expected_result)

    def test_info(self) -> None:
        epp_communicator = MagicMock(EppCommunicator)
        domain = Domain(epp_communicator)
        execute_result = (
            EppResultData(**{'client_transaction_id': None,
                             'code': 1000,
                             'message': 'Command completed successfully',
                             'raw_response': '<response>\n'
                                             '<result code="1000">\n'
                                             '<msg>Command completed successfully</msg>\n'
                                             '</result>\n'
                                             '<resData>\n'
                                             '<domain:infData>\n'
                                             '<domain:name>internet.nz</domain:name>\n'
                                             '<domain:roid>9204701-INZ</domain:roid>\n'
                                             '<domain:status s="inactive"/>\n'
                                             '<domain:registrant>inz-contact-1</domain:registrant>\n'
                                             '<domain:contact '
                                             'type="admin">inz-contact-1</domain:contact>\n'
                                             '<domain:contact '
                                             'type="tech">inz-contact-1</domain:contact>\n'
                                             '<domain:clID>933</domain:clID>\n'
                                             '<domain:crID>933</domain:crID>\n'
                                             '<domain:crDate>2023-02-23T21:56:22.713Z</domain:crDate>\n'
                                             '<domain:upID>CIRA_RAR_1</domain:upID>\n'
                                             '<domain:upDate>2023-02-24T21:59:27.901Z</domain:upDate>\n'
                                             '<domain:exDate>2024-02-23T21:56:22.713Z</domain:exDate>\n'
                                             '</domain:infData>\n'
                                             '</resData>\n'
                                             '<extension>\n'
                                             '<secDNS:infData '
                                             'xmlns:secDNS="urn:ietf:params:xml:ns:secDNS-1.1">\n'
                                             '<secDNS:dsData>\n'
                                             '<secDNS:keyTag>17600</secDNS:keyTag>\n'
                                             '<secDNS:alg>13</secDNS:alg>\n'
                                             '<secDNS:digestType>1</secDNS:digestType>\n'
                                             '<secDNS:digest>4ffb0f3f0fb5edbaa0067d1cd94bb545249cfea2</secDNS:digest>\n'
                                             '<secDNS:keyData>\n'
                                             '<secDNS:flags>257</secDNS:flags>\n'
                                             '<secDNS:protocol>3</secDNS:protocol>\n'
                                             '<secDNS:alg>14</secDNS:alg>\n'
                                             '<secDNS:pubKey>1fHiagyGUIxsDmeYaaKOWceffQn7QTaCwM1y6Oy1zYRpndxu/6WAHM99p/X/dM7t+RcGahZ8+GlX5JHkbyaMllg==</secDNS:pubKey>\n'
                                             '</secDNS:keyData>\n'
                                             '</secDNS:dsData>\n'
                                             '<secDNS:dsData>\n'
                                             '<secDNS:keyTag>17600</secDNS:keyTag>\n'
                                             '<secDNS:alg>13</secDNS:alg>\n'
                                             '<secDNS:digestType>2</secDNS:digestType>\n'
                                             '<secDNS:digest>024c96b094aaad3892babb303e02587f7d6f8f749fef9ff642e3bb8bfc2e0dcf</secDNS:digest>\n'
                                             '<secDNS:keyData>\n'
                                             '<secDNS:flags>257</secDNS:flags>\n'
                                             '<secDNS:protocol>3</secDNS:protocol>\n'
                                             '<secDNS:alg>13</secDNS:alg>\n'
                                             '<secDNS:pubKey>2fHiagyGUIxsDmeYaaKOWceffQn7QTaCwM1y6Oy1zYRpndxu/6WAHM99p/X/dM7t+RcGahZ8+GlX5JHkbyaMllg==</secDNS:pubKey>\n'
                                             '</secDNS:keyData>\n'
                                             '</secDNS:dsData>\n'
                                             '<secDNS:dsData>\n'
                                             '<secDNS:keyTag>17600</secDNS:keyTag>\n'
                                             '<secDNS:alg>13</secDNS:alg>\n'
                                             '<secDNS:digestType>4</secDNS:digestType>\n'
                                             '<secDNS:digest>14d92f9f1e82c0906e2003b9e3401de4024f705724ef14d848fc2708d5714d5572db040faa3e317753e6577c7e70cbb8</secDNS:digest>\n'
                                             '<secDNS:keyData>\n'
                                             '<secDNS:flags>257</secDNS:flags>\n'
                                             '<secDNS:protocol>3</secDNS:protocol>\n'
                                             '<secDNS:alg>13</secDNS:alg>\n'
                                             '<secDNS:pubKey>3fHiagyGUIxsDmeYaaKOWceffQn7QTaCwM1y6Oy1zYRpndxu/6WAHM99p/X/dM7t+RcGahZ8+GlX5JHkbyaMllg==</secDNS:pubKey>\n'
                                             '</secDNS:keyData>\n'
                                             '</secDNS:dsData>\n'
                                             '</secDNS:infData>\n'
                                             '<rgp:infData xmlns:rgp="urn:ietf:params:xml:ns:rgp-1.0">\n'
                                             '<rgp:rgpStatus s="addPeriod"/>\n'
                                             '</rgp:infData>\n'
                                             '<regtype:infData '
                                             'xmlns:regtype="urn:ietf:params:xml:ns:regtype-0.1">\n'
                                             '<regtype:type>standard</regtype:type>\n'
                                             '</regtype:infData>\n'
                                             '</extension>\n'
                                             '<trID>\n'
                                             '<svTRID>CIRA-000063163743-0000000003</svTRID>\n'
                                             '</trID>\n'
                                             '</response>',
                             'reason': None,
                             'repository_object_id': '9204701-INZ',
                             'server_transaction_id': 'CIRA-000063163743-0000000003',
                             'result_data': None}))

        expected_result = \
            EppResultData(**{'client_transaction_id': None,
                             'code': 1000,
                             'message': 'Command completed successfully',
                             'raw_response': '<response>\n'
                                             '<result code="1000">\n'
                                             '<msg>Command completed successfully</msg>\n'
                                             '</result>\n'
                                             '<resData>\n'
                                             '<domain:infData>\n'
                                             '<domain:name>internet.nz</domain:name>\n'
                                             '<domain:roid>9204701-INZ</domain:roid>\n'
                                             '<domain:status s="inactive"/>\n'
                                             '<domain:registrant>inz-contact-1</domain:registrant>\n'
                                             '<domain:contact '
                                             'type="admin">inz-contact-1</domain:contact>\n'
                                             '<domain:contact '
                                             'type="tech">inz-contact-1</domain:contact>\n'
                                             '<domain:clID>933</domain:clID>\n'
                                             '<domain:crID>933</domain:crID>\n'
                                             '<domain:crDate>2023-02-23T21:56:22.713Z</domain:crDate>\n'
                                             '<domain:upID>CIRA_RAR_1</domain:upID>\n'
                                             '<domain:upDate>2023-02-24T21:59:27.901Z</domain:upDate>\n'
                                             '<domain:exDate>2024-02-23T21:56:22.713Z</domain:exDate>\n'
                                             '</domain:infData>\n'
                                             '</resData>\n'
                                             '<extension>\n'
                                             '<secDNS:infData '
                                             'xmlns:secDNS="urn:ietf:params:xml:ns:secDNS-1.1">\n'
                                             '<secDNS:dsData>\n'
                                             '<secDNS:keyTag>17600</secDNS:keyTag>\n'
                                             '<secDNS:alg>13</secDNS:alg>\n'
                                             '<secDNS:digestType>1</secDNS:digestType>\n'
                                             '<secDNS:digest>4ffb0f3f0fb5edbaa0067d1cd94bb545249cfea2</secDNS:digest>\n'
                                             '<secDNS:keyData>\n'
                                             '<secDNS:flags>257</secDNS:flags>\n'
                                             '<secDNS:protocol>3</secDNS:protocol>\n'
                                             '<secDNS:alg>14</secDNS:alg>\n'
                                             '<secDNS:pubKey>1fHiagyGUIxsDmeYaaKOWceffQn7QTaCwM1y6Oy1zYRpndxu/6WAHM99p/X/dM7t+RcGahZ8+GlX5JHkbyaMllg==</secDNS:pubKey>\n'
                                             '</secDNS:keyData>\n'
                                             '</secDNS:dsData>\n'
                                             '<secDNS:dsData>\n'
                                             '<secDNS:keyTag>17600</secDNS:keyTag>\n'
                                             '<secDNS:alg>13</secDNS:alg>\n'
                                             '<secDNS:digestType>2</secDNS:digestType>\n'
                                             '<secDNS:digest>024c96b094aaad3892babb303e02587f7d6f8f749fef9ff642e3bb8bfc2e0dcf</secDNS:digest>\n'
                                             '<secDNS:keyData>\n'
                                             '<secDNS:flags>257</secDNS:flags>\n'
                                             '<secDNS:protocol>3</secDNS:protocol>\n'
                                             '<secDNS:alg>13</secDNS:alg>\n'
                                             '<secDNS:pubKey>2fHiagyGUIxsDmeYaaKOWceffQn7QTaCwM1y6Oy1zYRpndxu/6WAHM99p/X/dM7t+RcGahZ8+GlX5JHkbyaMllg==</secDNS:pubKey>\n'
                                             '</secDNS:keyData>\n'
                                             '</secDNS:dsData>\n'
                                             '<secDNS:dsData>\n'
                                             '<secDNS:keyTag>17600</secDNS:keyTag>\n'
                                             '<secDNS:alg>13</secDNS:alg>\n'
                                             '<secDNS:digestType>4</secDNS:digestType>\n'
                                             '<secDNS:digest>14d92f9f1e82c0906e2003b9e3401de4024f705724ef14d848fc2708d5714d5572db040faa3e317753e6577c7e70cbb8</secDNS:digest>\n'
                                             '<secDNS:keyData>\n'
                                             '<secDNS:flags>257</secDNS:flags>\n'
                                             '<secDNS:protocol>3</secDNS:protocol>\n'
                                             '<secDNS:alg>13</secDNS:alg>\n'
                                             '<secDNS:pubKey>3fHiagyGUIxsDmeYaaKOWceffQn7QTaCwM1y6Oy1zYRpndxu/6WAHM99p/X/dM7t+RcGahZ8+GlX5JHkbyaMllg==</secDNS:pubKey>\n'
                                             '</secDNS:keyData>\n'
                                             '</secDNS:dsData>\n'
                                             '</secDNS:infData>\n'
                                             '<rgp:infData xmlns:rgp="urn:ietf:params:xml:ns:rgp-1.0">\n'
                                             '<rgp:rgpStatus s="addPeriod"/>\n'
                                             '</rgp:infData>\n'
                                             '<regtype:infData '
                                             'xmlns:regtype="urn:ietf:params:xml:ns:regtype-0.1">\n'
                                             '<regtype:type>standard</regtype:type>\n'
                                             '</regtype:infData>\n'
                                             '</extension>\n'
                                             '<trID>\n'
                                             '<svTRID>CIRA-000063163743-0000000003</svTRID>\n'
                                             '</trID>\n'
                                             '</response>',
                             'repository_object_id': '9204701-INZ',
                             'result_data': DomainData(domain_name='internet.nz',
                                                       sponsoring_client_id='933',
                                                       status=['inactive'],
                                                       host=None,
                                                       registrant='inz-contact-1',
                                                       admin='inz-contact-1',
                                                       tech='inz-contact-1',
                                                       billing=None,
                                                       create_date='2023-02-23T21:56:22.713Z',
                                                       creat_client_id='933',
                                                       update_client_id='CIRA_RAR_1',
                                                       update_date='2023-02-24T21:59:27.901Z',
                                                       expiry_date='2024-02-23T21:56:22.713Z',
                                                       transfer_date=None,
                                                       password=None,
                                                       period=None,
                                                       dns_sec=[DSRecordData(key_tag='17600',
                                                                             algorithm='13',
                                                                             digest_type='1',
                                                                             digest='4ffb0f3f0fb5edbaa0067d1cd94bb545249cfea2',
                                                                             dns_key={'algorithm': '13',
                                                                                      'flag': '257',
                                                                                      'protocol': '3',
                                                                                      'public_key': '1fHiagyGUIxsDmeYaaKOWceffQn7QTaCwM1y6Oy1zYRpndxu/6WAHM99p/X/dM7t+RcGahZ8+GlX5JHkbyaMllg=='}),
                                                                DSRecordData(key_tag='17600',
                                                                             algorithm='13',
                                                                             digest_type='2',
                                                                             digest='024c96b094aaad3892babb303e02587f7d6f8f749fef9ff642e3bb8bfc2e0dcf',
                                                                             dns_key={'algorithm': '13',
                                                                                      'flag': '257',
                                                                                      'protocol': '3',
                                                                                      'public_key': '2fHiagyGUIxsDmeYaaKOWceffQn7QTaCwM1y6Oy1zYRpndxu/6WAHM99p/X/dM7t+RcGahZ8+GlX5JHkbyaMllg=='}),
                                                                DSRecordData(key_tag='17600',
                                                                             algorithm='13',
                                                                             digest_type='4',
                                                                             digest='14d92f9f1e82c0906e2003b9e3401de4024f705724ef14d848fc2708d5714d5572db040faa3e317753e6577c7e70cbb8',
                                                                             dns_key={'algorithm': '13',
                                                                                      'flag': '257',
                                                                                      'protocol': '3',
                                                                                      'public_key': '3fHiagyGUIxsDmeYaaKOWceffQn7QTaCwM1y6Oy1zYRpndxu/6WAHM99p/X/dM7t+RcGahZ8+GlX5JHkbyaMllg=='})]
                                                       ),
                             'server_transaction_id': 'CIRA-000063163743-0000000003'})

        domain.execute = MagicMock(return_value=execute_result)

        result = domain.info('internet.nz')
        self.assertEqual(result, expected_result)

    def test_create(self) -> None:
        expected_result = (
            EppResultData(**{'client_transaction_id': '96aaf073-5741-47bd-b1eb-b5abcf4206fa',
                             'code': 1000,
                             'message': 'Command completed successfully',
                             'raw_response': '<response>\n'
                                             '<result code="1000">\n'
                                             '<msg>Command completed successfully</msg>\n'
                                             '</result>\n'
                                             '<resData>\n'
                                             '<domain:creData>\n'
                                             '<domain:name>internet.nz</domain:name>\n'
                                             '<domain:crDate>2023-05-07T23:36:25.681Z</domain:crDate>\n'
                                             '<domain:exDate>2026-05-07T23:36:25.681Z</domain:exDate>\n'
                                             '</domain:creData>\n'
                                             '</resData>\n'
                                             '<trID>\n'
                                             '<clTRID>96aaf073-5741-47bd-b1eb-b5abcf4206fa</clTRID>\n'
                                             '<svTRID>CIRA-000064317159-0000000003</svTRID>\n'
                                             '</trID>\n'
                                             '</response>',
                             'reason': None,
                             'repository_object_id': None,
                             'server_transaction_id': 'CIRA-000064317159-0000000003',
                             'result_data': None}))

        create_params = DomainData(
            domain_name='internet.nz',
            registrant='inz-contact-3',
            admin='inz-contact-1',
            tech='inz-contact-1',
            billing='inz-contact-3',
            period=3,
            host=['01y.test-indwrx2vkicn2otgm3otav5wpnzvjd.co.nz', '0d9x6239.drdomain.co.nz'],
            dns_sec=DSRecordData(
                key_tag=1235,
                algorithm=DNSSECAlgorithm.DSA_SHA_1.value,
                digest_type=DigestTypeEnum.SHA_1.value,
                digest='8cdb09364147aed879d12c68d615f98af5900b73'
            ),
        )
        epp_communicator = MagicMock(EppCommunicator)
        domain = Domain(epp_communicator)
        domain.execute = MagicMock(return_value=expected_result)

        result = domain.create(create_params)

        self.assertEqual(result, expected_result)

    def test_delete(self) -> None:
        expected_result = EppResultData(**{'client_transaction_id': 'b9c0c77d-b2d9-47c7-9edb-b4d4988c2e7d',
                                           'code': 1000,
                                           'message': 'Command completed successfully',
                                           'raw_response': '<response>\n'
                                                           '<result code="1000">\n'
                                                           '<msg>Command completed successfully</msg>\n'
                                                           '</result>\n'
                                                           '<trID>\n'
                                                           '<clTRID>b9c0c77d-b2d9-47c7-9edb-b4d4988c2e7d</clTRID>\n'
                                                           '<svTRID>CIRA-000064688525-0000000004</svTRID>\n'
                                                           '</trID>\n'
                                                           '</response>',
                                           'reason': None,
                                           'repository_object_id': None,
                                           'server_transaction_id': 'CIRA-000064688525-0000000004',
                                           'result_data': None})

        epp_communicator = MagicMock(EppCommunicator)
        domain = Domain(epp_communicator)
        domain.execute = MagicMock(return_value=expected_result)

        result = domain.delete('internet.nz')

        self.assertEqual(result, expected_result)

    def test_renew(self) -> None:
        expected_result = EppResultData(
            **{'client_transaction_id': '3fc04fc6-0b40-4979-99c0-fa91e9da573f',
               'code': 1000,
               'message': 'Command completed successfully',
               'raw_response': '<response>\n'
                               '<result code="1000">\n'
                               '<msg>Command completed successfully</msg>\n'
                               '</result>\n'
                               '<resData>\n'
                               '<domain:renData>\n'
                               '<domain:name>internet.nz</domain:name>\n'
                               '<domain:exDate>2026-01-21T21:56:22.713Z</domain:exDate>\n'
                               '</domain:renData>\n'
                               '</resData>\n'
                               '<trID>\n'
                               '<clTRID>3fc04fc6-0b40-4979-99c0-fa91e9da573f</clTRID>\n'
                               '<svTRID>CIRA-000065618145-0000000004</svTRID>\n'
                               '</trID>\n'
                               '</response>',
               'reason': None,
               'repository_object_id': None,
               'server_transaction_id': 'CIRA-000065618145-0000000004',
               'result_data': None})

        epp_communicator = MagicMock(EppCommunicator)
        domain = Domain(epp_communicator)
        domain.execute = MagicMock(return_value=expected_result)

        result = domain.renew(domain_name='internet.nz', expiry_date=date(2024, 1, 1), period=2)

        self.assertEqual(result, expected_result)

    def test_transfer(self) -> None:
        expected_result = EppResultData(
            **{'client_transaction_id': '3fc04fc6-0b40-4979-99c0-fa91e9da573f',
               'code': 1000,
               'message': 'Command completed successfully',
               'raw_response': '<response>\n'
                               '<result code="1000">\n'
                               '<msg>Command completed successfully</msg>\n'
                               '</result>\n'
                               '<resData>\n'
                               '<domain:renData>\n'
                               '<domain:name>internet.nz</domain:name>\n'
                               '<domain:reID>ClientX</domain:reID>\n'
                               '<domain:reDate>2000-06-08T22:00:00.0Z</domain:reDate>\n'
                               '<domain:acID>ClientY</domain:acID>\n'
                               '<domain:acDate>2000-06-13T22:00:00.0Z</domain:acDate>\n'
                               '<domain:exDate>2002-09-08T22:00:00.0Z</domain:exDate>\n'
                               '</domain:renData>\n'
                               '</resData>\n'
                               '<trID>\n'
                               '<clTRID>3fc04fc6-0b40-4979-99c0-fa91e9da573f</clTRID>\n'
                               '<svTRID>CIRA-000065618145-0000000004</svTRID>\n'
                               '</trID>\n'
                               '</response>',
               'reason': None,
               'repository_object_id': None,
               'server_transaction_id': 'CIRA-000065618145-0000000004',
               'result_data': None})

        epp_communicator = MagicMock(EppCommunicator)
        domain = Domain(epp_communicator)
        domain.execute = MagicMock(return_value=expected_result)

        result = domain.transfer(domain_name='internet.nz', password='PassWord', period=1)

        self.assertEqual(result, expected_result)

    def test_domain_update(self) -> None:
        expected_result = EppResultData(
            **{'client_transaction_id': 'd2066d60-fc5e-4b7a-9001-893ca2957857',
               'code': 1000,
               'message': 'Command completed successfully',
               'raw_response': '<response>\n'
                               '<result code="1000">\n'
                               '<msg>Command completed successfully</msg>\n'
                               '</result>\n'
                               '<trID>\n'
                               '<clTRID>d2066d60-fc5e-4b7a-9001-893ca2957857</clTRID>\n'
                               '<svTRID>CIRA-000068457178-0000000004</svTRID>\n'
                               '</trID>\n'
                               '</response>',
               'reason': None,
               'repository_object_id': None,
               'server_transaction_id': 'CIRA-000068457178-0000000004',
               'result_data': None})

        epp_communicator = MagicMock(EppCommunicator)
        domain = Domain(epp_communicator)
        domain.execute = MagicMock(return_value=expected_result)
        domain.info = MagicMock(return_value={
            'result_data': DomainData(
                domain_name='internet.nz',
                registrant='registrant1',
                admin='old-admin',
                tech='old-tech',
                period=1
            )
        })

        result = domain.update(domain_name='internet.nz',
                               password='PassWord',
                               add_admins=['contact-admin',],
                               add_techs=['contact-tech',])

        self.assertEqual(result, expected_result)
